/*
**            SHOWDATA
**  Displays data received on a comm port.  Useful as a line monitor
**  between two other devices.
**
**  Command line:
**    showdata [options]
**  where [options] are:
**    -p[1,2,3,4]     set port 1-4 (default is 1)
**    -s[300 1200 2400 4800 9600, etc.]   set baud rate (default is 9600)
**    filename    dump to file
**    -a          start in ascii mode (default is binary)
**
** Copyright 1989, Steven E. Margison
** modified 1989 by Bob Stout for ZTC using CXL
**
** This program is NOT public domain, but is included with
**   "Steve's Library for Turbo-C"
**               &
**   "Steve's Library for Zortech-C"
** as a helpful utility and demonstration of the async functions.
**
** while running:
**   ESC quits program
**   FK1 opens a dump file
**   FK2 closes a dump flie
**   FK3 pauses data collection
**   FK4 un-pauses data collection
**   FK5 enables ASCII mode
**   FK6 disables ASCII mode
**   FK7 strips bit 7
**   FK8 allows bit 7
**
**  In binary (non-ascii) mode, each received byte is displayed as 2 hex
**  digits.
**  In ASCII mode, each printable ASCII character from 20h to 7eh is displayed
**  as itself.  Controls characters (less than 20h) are displayed as their
**  standard name, in lower case (i.e., nul stx bel dc1 etc.).
**  All characters 7fh or above are always displayed as binary.
**
**  NOTE:  This program requires the "CXL" windowing library, by Mike
**         Smedley.  Evaluation copies of this excellent shareware
**         windows package are available from many sources.  The windowing
**         functions are used only for filename entry, and can be re-written
**         if desired.
*/

/* 5-8-89 */

#include <stdio.h>
#include <dos.h>
#include <mflsys.h>
#include <mflconio.h>
#include <cxlwin.h>

FILE *fd;

static int      ascii,         /* TRUE if ascii mode */
                dumping,       /* TRUE if dumping to file */
                speed,         /* baud rate value */
                port,          /* port number */
                pause,         /* TRUE to holdoff more data */
                col,           /* current position on screen */
                row,           /*   "        "     "    "    */
                in_str,
                strip7;        /* strip bit 7 if set */

void doargs(int, char*[]), do_file(void), bump_row(void);

char *strings[32] = { " nul",
                     " soh",
                     " stx",
                     " etx",
                     " eot",
                     " eng",
                     " ack",
                     " bel",
                     " bs ",
                     " ht ",
                     " lf ",
                     " vt ",
                     " ff ",
                     " cr ",
                     " so ",
                     " si ",
                     " dle",
                     " dc1",
                     " dc2",
                     " dc3",
                     " dc4",
                     " nak",
                     " syn",
                     " etb",
                     " can",
                     " em ",
                     " sub",
                     " esc",
                     " fs ",
                     " gs ",
                     " rs ",
                     " vs " };

main(argc, argv) int argc; char *argv[];
{
        int i;
        char tbuf[20];

        strip7 = in_str = pause = ascii = dumping = FALSE;
        speed = 9600;
        port = SER1;
        doargs(argc, argv);
        if(!config_port(speed, NONE, 8, TRUE, port))
        {
                if(dumping)
                        fclose(fd);
                error("Cannot initialize port");
        }

        cls();
        disp_open();    /* use direct video access */
        sprintf(tbuf, "Port: %d   Speed: %d", port+1, speed);
        disp_say(23, 0, tbuf);
        if(ascii)
                disp_say(23, 25, "ASCII");
        disp_move(0,0);
        disp_flush();
        row = col = 0;
        tbuf[3] = NULL;

        for EVER
        {
                if(kbhit())
                {
                        i = getkey();
                        if(i == ESC)
                        {
                                if(dumping)
                                {
                                        fputs("\n\n", fd);
                                        fclose(fd);
                                }
                                disp_close();
                                cls();
                                exit();
                        }
                        if(i > 0xff)
                        {
                                switch(i - 256)
                                {
                                case FK1:
                                        if(!dumping)
                                                do_file();
                                        break;
                                case FK2:
                                        if(dumping)
                                        {
                                                fputs("\n\n", fd);
                                                fclose(fd);
                                                dumping = FALSE;
                                                disp_sayr(23, 46, "                                ");
                                        }
                                        break;
                                case FK3:
                                        pause = TRUE;
                                        disp_sayr(23, 31, "PAUSED");
                                        break;
                                case FK4:
                                        pause = FALSE;
                                        disp_sayr(23, 31, "      ");
                                        break;
                                case FK5:
                                        ascii = TRUE;
                                        disp_sayr(23, 25, "ASCII");
                                        break;
                                case FK6:
                                        ascii = FALSE;
                                        disp_sayr(23, 25, "     ");
                                        break;
                                case FK7:
                                        strip7 = TRUE;
                                        disp_sayr(23, 38, "STRIP7");
                                        break;
                                case FK8:
                                        strip7 = FALSE;
                                        disp_sayr(23, 38, "      ");
                                        break;
                                default:
                                        break;    /* ignore everything else */
                                }  /* end of switch */
                        }
                }  /* end of keyboard process */

                if(ready_rcv(port) && !pause)
                {
                        i = readchar(port);
                        if(strip7)
                                i &= 0x7f;
                        if((i > 0x7e) || !ascii)
                        {
                                tbuf[0] = ' ';
                                tbuf[1] = ((i >> 4) & 0x0f) + '0';
                                if(tbuf[1] > '9')
                                        tbuf[1] += 7;
                                tbuf[2] = (i & 0x0f) + '0';
                                if(tbuf[2] > '9')
                                        tbuf[2] += 7;
                                disp_puts(tbuf);
                                if(dumping)
                                        fputs(tbuf, fd);
                                col += 3;
                                if(col > 77)
                                        bump_row();
                        }   /* end of binary display path */
                        else
                        {
                                if(i < ' ')
                                {
                                        in_str = FALSE;
                                        disp_puts(strings[i]);
                                        if(dumping)
                                                fputs(strings[i], fd);
                                        col += 4;
                                        if(col > 77)
                                                bump_row();
                                }
                                else
                                {
                                        if(!in_str)
                                        {
                                                in_str = TRUE;
                                                disp_putc(' ');
                                                if(dumping)
                                                        fputc(' ', fd);
                                                col++;
                                        }
                                        disp_putc(i);
                                        if(dumping)
                                                fputc(i, fd);
                                        col++;
                                        if(col > 77)
                                                bump_row();
                                }
                        }
                        disp_flush();
                }  /* end of port receive */

        }  /* end of for ever loop */

}    /* end of main */

void doargs(argc, argv)
int argc;
char *argv[];
{
        if(argc < 2)
                return;
        fd = NULL;
        while(--argc > 0)
        {
                if(argv[argc][0] != '-')
                {
                        if(dumping)
                        {
                                fclose(fd);
                                error("Only one output file allowed.");
                        }
                        else
                        {
                                if((fd = fopen(argv[argc], "a")) == NULL)
                                {
                                        cant(argv[argc]);
                                }
                                dumping = TRUE;
                        }
                }
                else
                {
                        switch(argv[argc][1])
                        {
                        case 's':
                        case 'S':
                                speed = atoi(&argv[argc][2]);
                                break;
                        case 'p':
                        case 'P':
                                port = (argv[argc][2] & 0x0f) - 1;
                                if((port < 0) || (port > 3))
                                {
                                        error("Invalid port number.");
                                }
                                break;
                        case 'a':
                        case 'A':
                                ascii = TRUE;
                                break;
                        default:
                                error("Invalid option");
                                break;
                        }  /* end of switch */
                }
        }  /* end of while */
}


void do_file()
{
        WINDOW wn;
        char filename[MAX_FLEN];

        if(0 == (wn = wopen(10, 5, 12, 75, 1, _BLUE|WHITE, _BLUE|YELLOW)))
                return;
        wtitle("ENTER CAPTURE FILENAME", TCENTER, _BLUE|WHITE);

        winputsf(filename, "' ' !U! *********************************"
                "*********************************");
        if (!filename[0] || NULL == (fd = fopen(filename, "a")))
        {
                wtitle("Cannot open that file.",
                        TCENTER, _BLUE|WHITE);
                waitkey();
                disp_say(23, 46, "                              ");
        }
        else
        {
                dumping = TRUE;
                disp_say(23, 46, filename);
        }
        wcloseall();
        disp_flush();
        disp_move(row, col);
        disp_flush();
}


void bump_row()
{
        if(dumping)
                fputc('\n', fd);
        col = 0;
        if(++row > 22)
        {
                row = 22;
                disp_scroll(1, 0, 0, 22, 79, NORMAL);
                disp_clrfield(22, 0, 80);
        }
        disp_move(row, 0);
        disp_flush();
}
